/*
  Copyright (C) 2009, 2010 and 2013 Chris Vine

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef MOUNT_ENTRY_H
#define MOUNT_ENTRY_H

#include "prog_defs.h"

#include <string>
#include <utility>

#include <gtk/gtk.h>

#include <c++-gtk-utils/window.h>
#include <c++-gtk-utils/gobj_handle.h>

namespace MountEntriesCB {
  extern "C" {
    void mount_entries_button_clicked(GtkWidget*, void*);
    void mount_entries_drag_n_drop(GtkTreeModel*, GtkTreePath*, void*);
  }
}

class TableEntry;

class MountEntries: public WinBase {

  bool changed;

  GtkWidget* ok_button_p;
  GtkWidget* cancel_button_p;
  GtkWidget* add_button_p;
  GtkWidget* modify_button_p;
  GtkWidget* delete_button_p;
  GtkWidget* up_button_p;
  GtkWidget* down_button_p;

  GobjHandle<GtkTreeModel> list_store_h;
  
  GtkTreeView* tree_view_p;

  void read_list(void);
  void save_list(void);
  void finish(void);
  void add_entry_prompt(void);
  void add_entry(const TableEntry&);
  void modify_entry_prompt(void);
  void modify_entry(const TableEntry&);
  void delete_entry_prompt(void);
  void delete_entry(void);
  void move_up_impl(void);
  void move_down_impl(void);

  // Objects of this class are not for copying
  MountEntries(const MountEntries&);
  MountEntries& operator=(const MountEntries&);
protected:
  virtual int get_exec_val(void) const;
  virtual void on_delete_event(void);
public:
  friend void MountEntriesCB::mount_entries_button_clicked(GtkWidget*, void*);
  friend void MountEntriesCB::mount_entries_drag_n_drop(GtkTreeModel*, GtkTreePath*, void*);

  MountEntries(GtkWindow* parent_p);
};

namespace TableEntryCB {
  extern "C" {
    void new_entry_selected(GtkWidget*, void*);
    gboolean new_entry_key_press_event(GtkWidget*, GdkEventKey*, void*);
  }
}

class TableEntry: public WinBase {

  bool accepted;

  GtkWidget* ok_button_p;
  GtkWidget* cancel_button_p;
  GtkWidget* device_entry_p;
  GtkWidget* label_entry_p;

  std::pair<std::string, std::string> result;

  bool selected_impl(void);

  // Objects of this class are not for copying
  TableEntry(const TableEntry&);
  TableEntry& operator=(const TableEntry&);
protected:
  virtual int get_exec_val(void) const;
public:
  friend void TableEntryCB::new_entry_selected(GtkWidget*, void*);
  friend gboolean TableEntryCB::new_entry_key_press_event(GtkWidget*, GdkEventKey*, void*);

  std::pair<std::string, std::string> get_result(void) const
    {return result;}

  // the last two parameters are NULL if a new entry is being created,
  // otherwise it is an entry to be modified
  TableEntry(GtkWindow* parent_p, const gchar* device_p, const gchar* label_p);
};

#endif
